#--
# Copyright 2006 by Chad Fowler, Rich Kilmer, Jim Weirich and others.
# All rights reserved.
# See LICENSE.txt for permissions.
#++

class Gem::Ext::Builder

  def self.class_name
    name =~ /Ext::(.*)Builder/
    $1.downcase
  end

  def self.get_status_of_forked_command(command, results)
    read_size = 0

    results << command
    read_size += command.size
    puts "#{command}" if Gem.configuration.really_verbose

    iopipe_r, iopipe_w = IO.pipe

    child_pid = fork{
      # Child
      $stdin.close
      iopipe_r.close
      $stdout.reopen(iopipe_w)
      #  redirect
      $stderr.reopen(iopipe_w)

      exec "#{command}"

      puts "Executing #{command} failed."
      exit 127
    }

    iopipe_w.close
    output = ""

    if (child_pid < 0) then
      raise Gem::InstallError, "Forking process of #{command} failed\n"
    end

    iopipe_r.each{|line|
      output += line
      if Gem.configuration.really_verbose then
        puts "#{line}" if Gem.configuration.really_verbose
      else
        read_size += line.size
        while read_size >= 128 do
          print "."
          $stdout.flush
          read_size -= 128
        end
      end
    }
    results << output
    print "\n" if not Gem.configuration.really_verbose

    child_pid, status = Process.waitpid2(child_pid)
    return status
  end

  def self.make(dest_path, results)
    unless File.exist? 'Makefile' then
      raise Gem::InstallError, "Makefile not found:\n\n#{results.join "\n"}"
    end

    mf = File.read('Makefile')
    mf = mf.gsub(/^RUBYARCHDIR\s*=\s*\$[^$]*/, "RUBYARCHDIR = #{dest_path}")
    mf = mf.gsub(/^RUBYLIBDIR\s*=\s*\$[^$]*/, "RUBYLIBDIR = #{dest_path}")

    File.open('Makefile', 'wb') {|f| f.print mf}

    # try to find make program from Ruby configure arguments first
    RbConfig::CONFIG['configure_args'] =~ /with-make-prog\=(\w+)/
    make_program = $1 || ENV['make']
    unless make_program then
      make_program = (/mswin/ =~ RUBY_PLATFORM) ? 'nmake' : 'make'
    end

    ['', ' install'].each do |target|
      cmd = "#{make_program}#{target}"
      status = self.get_status_of_forked_command(cmd, results)

      raise Gem::InstallError, "make#{target} failed:\n\n#{results}" unless
        status.success?
    end
  end

  def self.redirector
    '2>&1'
  end

  def self.run(command, results)
    status = self.get_status_of_forked_command(command, results)
    unless status.success? then
      raise Gem::InstallError, "#{class_name} failed:\n\n#{results.join "\n"}"
    end
  end

end

